@@ -18,7 +18,7 @@
   twm and enlightenment.
 */
 
-/* ---------------------------- included header files ----------------------- */
+/* ---------------------------- included header files ---------------------- */
 
 #include "config.h"
 
@@ -28,10 +28,8 @@
 #endif
 #include <signal.h>
 #include <fcntl.h>
-#ifdef SESSION
-#include <X11/SM/SMlib.h>
-#endif
 
+#include "libs/FSMlib.h"
 #include "libs/fvwmlib.h"
 #include "fvwm.h"
 #include "externs.h"
@@ -47,27 +45,26 @@
 #include "geometry.h"
 #include "move_resize.h"
 
-/* ---------------------------- local definitions --------------------------- */
+/* ---------------------------- local definitions -------------------------- */
 
-/*#define FVWM_DEBUG_MSGS 1*/
-/*#define FVWM_DEBUG_DEVEL 1*/
+/*#define FVWM_SM_DEBUG_PROTO*/
+/*#define FVWM_SM_DEBUG_WINMATCH*/
+/*#define FVWM_SM_DEBUG_FILES*/
 
-/* ---------------------------- local macros -------------------------------- */
+/* ---------------------------- local macros ------------------------------- */
 
 #define xstreq(a,b) ((!a && !b) || (a && b && (strcmp(a,b)==0)))
 
-/* ---------------------------- imports ------------------------------------- */
+/* ---------------------------- imports ------------------------------------ */
 
 extern Bool Restarting;
-#ifdef SESSION
 extern int master_pid;
 extern char **g_argv;
 extern int g_argc;
-#endif
 
-/* ---------------------------- included code files ------------------------- */
+/* ---------------------------- included code files ------------------------ */
 
-/* ---------------------------- local types --------------------------------- */
+/* ---------------------------- local types -------------------------------- */
 
 typedef struct _match
 {
@@ -91,31 +88,28 @@ typedef struct _match
 	window_flags flags;
 } Match;
 
-/* ---------------------------- forward declarations ------------------------ */
+/* ---------------------------- forward declarations ----------------------- */
 
-/* ---------------------------- local variables ----------------------------- */
+/* ---------------------------- local variables ---------------------------- */
 
-#ifdef SESSION
 static char *previous_sm_client_id = NULL;
 static char *sm_client_id = NULL;
 static Bool sent_save_done = 0;
 static char *realStateFilename = NULL;
 static Bool goingToRestart = False;
-static IceIOErrorHandler prev_handler;
-static SmcConn sm_conn = NULL;
-#endif
+static FIceIOErrorHandler prev_handler;
+static FSmcConn sm_conn = NULL;
+
 static int num_match = 0;
 static Match *matches = NULL;
 static Bool does_file_version_match = False;
 static Bool doPreserveState = True;
 
-/* ---------------------------- exported variables (globals) ---------------- */
+/* ---------------------------- exported variables (globals) --------------- */
 
-#ifdef SESSION
 int sm_fd = -1;
-#endif
 
-/* ---------------------------- local functions ----------------------------- */
+/* ---------------------------- local functions ---------------------------- */
 
 static
 char *duplicate(const char *s)
@@ -179,19 +173,22 @@ SaveGlobalState(FILE *f)
 	fprintf(f, "  [SCROLL] %i %i %i %i %i %i\n",
 		Scr.EdgeScrollX, Scr.EdgeScrollY, Scr.ScrollResistance,
 		Scr.MoveResistance,
-		!!(Scr.flags.edge_wrap_x), !!(Scr.flags.edge_wrap_y));
+		!!(Scr.flags.do_edge_wrap_x), !!(Scr.flags.do_edge_wrap_y));
 	fprintf(f, "  [SNAP] %i %i %i %i\n",
 		Scr.SnapAttraction, Scr.SnapMode, Scr.SnapGridX, Scr.SnapGridY);
 	fprintf(f, "  [MISC] %i %i %i\n",
 		Scr.ClickTime, Scr.ColormapFocus, Scr.ColorLimit);
-	fprintf(f, "  [STYLE] %i %i\n", Scr.gs.EmulateMWM, Scr.gs.EmulateWIN);
+	fprintf(f, "  [STYLE] %i %i\n", Scr.gs.do_emulate_mwm, Scr.gs.do_emulate_win);
 
 	return 1;
 }
 
-#ifdef SESSION
 static void setRealStateFilename(char *filename)
 {
+	if (!SessionSupport)
+	{
+		return;
+	}
 	if (realStateFilename)
 	{
 		free(realStateFilename);
@@ -207,6 +204,11 @@ static char *getUniqueStateFilename(void)
 	char *filename;
 	int fd;
 
+	if (!SessionSupport)
+	{
+		return NULL;
+	}
+
 	if (!path)
 	{
 		path = getenv ("HOME");
@@ -228,9 +230,9 @@ static char *getUniqueStateFilename(void)
 	{
 		return NULL;
 	}
-
-	filename = tempnam(path, ".fs-");
-	if ((fd = open (filename, O_WRONLY | O_EXCL | O_CREAT, 0600)) == -1)
+	filename = safestrdup(CatString2(path, "/.fs-XXXXXX"));
+	fd = fvwm_mkstemp(filename);
+	if (fd == -1)
 	{
 		free (filename);
 		filename = NULL;
@@ -242,9 +244,6 @@ static char *getUniqueStateFilename(void)
 
 	return filename;
 }
-#else
-#define setRealStateFilename(filename)
-#endif /* SESSION */
 
 static char *
 GetWindowRole(Window window)
@@ -259,15 +258,24 @@ GetWindowRole(Window window)
 			return ((char *) tp.value);
 		}
 	}
+	if (XGetTextProperty (dpy, window, &tp, _XA_WINDOW_ROLE))
+	{
+		if (tp.encoding == XA_STRING && tp.format == 8 &&
+		    tp.nitems != 0)
+		{
+			return ((char *) tp.value);
+		}
+	}
 
 	return NULL;
 }
 
 static char *
-GetClientID(Window window)
+GetClientID(FvwmWindow *fw)
 {
 	char *client_id = NULL;
-	Window client_leader;
+	Window client_leader = None;
+	Window window;
 	XTextProperty tp;
 	Atom actual_type;
 	int actual_format;
@@ -275,6 +283,8 @@ GetClientID(Window window)
 	unsigned long bytes_after;
 	unsigned char *prop = NULL;
 
+	window = FW_W(fw);
+
 	if (XGetWindowProperty(
 		    dpy, window, _XA_WM_CLIENT_LEADER, 0L, 1L, False,
 		    AnyPropertyType, &actual_type, &actual_format, &nitems,
@@ -284,22 +294,30 @@ GetClientID(Window window)
 		    nitems == 1 && bytes_after == 0)
 		{
 			client_leader = *((Window *) prop);
+		}
+	}
 
-			if (XGetTextProperty(
-				    dpy, client_leader, &tp, _XA_SM_CLIENT_ID))
+	if (!client_leader && fw->wmhints &&
+	    (fw->wmhints->flags & WindowGroupHint))
+	{
+		client_leader = fw->wmhints->window_group;
+	}
+
+	if (client_leader)
+	{
+		if (XGetTextProperty(dpy, client_leader, &tp, _XA_SM_CLIENT_ID))
+		{
+			if (tp.encoding == XA_STRING && tp.format == 8 &&
+			    tp.nitems != 0)
 			{
-				if (tp.encoding == XA_STRING &&
-				    tp.format == 8 && tp.nitems != 0)
-				{
-					client_id = (char *) tp.value;
-				}
+				client_id = (char *) tp.value;
 			}
 		}
+	}
 
-		if (prop)
-		{
-			XFree (prop);
-		}
+	if (prop)
+	{
+		XFree (prop);
 	}
 
 	return client_id;
@@ -388,9 +406,40 @@ SaveWindowStates(FILE *f)
 		}
 		is_icon_sticky_across_pages =
 			is_window_sticky_across_pages(ewin);
-		fprintf(f, "[CLIENT] %lx\n", FW_W(ewin));
 
-		client_id = GetClientID(FW_W(ewin));
+		wm_command = NULL;
+		wm_command_count = 0;
+
+		client_id = GetClientID(ewin);
+		if (!client_id)
+		{
+			/* no client id, some session manager do not manage
+			 * such client ... this can cause problem */
+			if (XGetCommand(
+				    dpy, FW_W(ewin), &wm_command,
+				    &wm_command_count) &&
+			    wm_command && wm_command_count > 0)
+			{
+				/* ok */
+			}
+			else
+			{
+				/* No client id and no WM_COMMAND, the client
+				 * cannot be managed by the sessiom manager
+				 * skip it! */
+				if (!Restarting)
+				{
+					if (wm_command)
+					{
+						XFreeStringList(wm_command);
+						wm_command = NULL;
+					}
+					continue;
+				}
+			}
+		}
+
+		fprintf(f, "[CLIENT] %lx\n", FW_W(ewin));
 		if (client_id)
 		{
 			fprintf(f, "  [CLIENT_ID] %s\n", client_id);
@@ -421,12 +470,7 @@ SaveWindowStates(FILE *f)
 					ewin->name.name);
 			}
 
-			wm_command = NULL;
-			wm_command_count = 0;
-			if (XGetCommand(
-				    dpy, FW_W(ewin), &wm_command,
-				    &wm_command_count) &&
-			    wm_command && wm_command_count > 0)
+			if (wm_command && wm_command_count > 0)
 			{
 				fprintf(f, "  [WM_COMMAND] %i",
 					wm_command_count);
@@ -437,13 +481,14 @@ SaveWindowStates(FILE *f)
 				}
 				fprintf(f, "\n");
 			}
-			if (wm_command)
-			{
-				XFreeStringList(wm_command);
-				wm_command = NULL;
-			}
 		} /* !window_role */
 
+		if (wm_command)
+		{
+			XFreeStringList(wm_command);
+			wm_command = NULL;
+		}
+
 		gravity_get_naked_geometry(
 			ewin->hints.win_gravity, ewin, &save_g,
 			&ewin->normal_g);
@@ -495,7 +540,7 @@ static Bool matchWin(FvwmWindow *w, Match *m)
 	int found;
 
 	found = 0;
-	client_id = GetClientID(FW_W(w));
+	client_id = GetClientID(w);
 
 	if (xstreq(client_id, m->client_id))
 	{
@@ -504,70 +549,103 @@ static Bool matchWin(FvwmWindow *w, Match *m)
 
 		window_role = GetWindowRole(FW_W(w));
 
-		if (window_role || m->window_role)
+		if (client_id && (window_role || m->window_role))
 		{
 			/* We have or had a window role, base decision on it */
-
 			found = xstreq(window_role, m->window_role);
 		}
-		else
+		else if (xstreq(w->class.res_name, m->res_name) &&
+			 xstreq(w->class.res_class, m->res_class) &&
+			 (IS_NAME_CHANGED(w) || IS_NAME_CHANGED(m) ||
+			  xstreq(w->name.name, m->wm_name)))
 		{
-			/* Compare res_class, res_name and WM_NAME, unless the
-			   user has changed WM_NAME */
-
-			if (xstreq(w->class.res_name, m->res_name) &&
-			    xstreq(w->class.res_class, m->res_class) &&
-			    (IS_NAME_CHANGED(w) || IS_NAME_CHANGED(m) ||
-			     xstreq(w->name.name, m->wm_name)))
+			if (client_id)
 			{
-				if (client_id)
+				/* If we have a client_id, we don't
+				 * compare WM_COMMAND, since it will be
+				 * different. */
+				found = 1;
+			}
+			else
+			{
+				/* for non-SM-aware clients we also
+				 * compare WM_COMMAND */
+
+				if (!XGetCommand(
+					    dpy, FW_W(w), &wm_command,
+					    &wm_command_count))
 				{
-					/* If we have a client_id, we don't
-					 * compare WM_COMMAND, since it will be
-					 * different. */
-					found = 1;
+					wm_command = NULL;
+					wm_command_count = 0;
 				}
-				else
+				if (wm_command_count == m->wm_command_count)
 				{
-					/* for non-SM-aware clients we also
-					 * compare WM_COMMAND */
-
-					if (!XGetCommand(
-						    dpy, FW_W(w), &wm_command,
-						    &wm_command_count))
+					for (i = 0; i < wm_command_count; i++)
 					{
-						wm_command = NULL;
-						wm_command_count = 0;
+						if (strcmp(unspace_string(
+								   wm_command[i]),
+							   m->wm_command[i])!=0)
+						{
+							break;
+						}
 					}
-					if (wm_command_count ==
-					    m->wm_command_count)
-					{
-						for (i = 0;
-						     i < wm_command_count; i++)
-						{
-							if (strcmp(unspace_string(wm_command[i]), m->wm_command[i]) != 0)
-							{
-								break;
-							}
-						}
 
-						if (i == wm_command_count)
+					if (i == wm_command_count)
+					{
+						/* migo (21/Oct/1999):
+						 * on restarts compare
+						 * window ids too */
+						if (!Restarting ||
+						    FW_W(w) == m->win)
 						{
-							/* migo (21/Oct/1999):
-							 * on restarts compare
-							 * window ids too */
-							if (!Restarting ||
-							    FW_W(w) == m->win)
-							{
-								found = 1;
-							}
+							found = 1;
 						}
-					} /* if (wm_command_count ==... */
-				} /* else no client id */
-			} /* if res_class, res_name and wm_name agree */
+					}
+				} /* if (wm_command_count ==... */
+			} /* else if res_class, res_name and wm_name agree */
 		} /* else no window roles */
 	} /* if client_id's agree */
 
+#ifdef FVWM_SM_DEBUG_WINMATCH
+	fprintf(stderr,
+		"\twin(%s, %s, %s, %s, %s,",
+		w->class.res_name, w->class.res_class, w->name.name,
+		(client_id)? client_id:"(null)",
+		(window_role)? window_role:"(null)");
+	if (wm_command)
+	{
+		for (i = 0; i < wm_command_count; i++)
+		{
+			fprintf(stderr," %s", wm_command[i]);
+		}
+		fprintf(stderr,",");
+	}
+	else
+	{
+		fprintf(stderr," no_wmc,");
+	}
+	fprintf(stderr," %d)", IS_NAME_CHANGED(w));
+	fprintf(stderr,"\n[%d]", found);
+	fprintf(stderr,
+		"\tmat(%s, %s, %s, %s, %s,",
+		m->res_name, m->res_class, m->wm_name,
+		(m->client_id)?m->client_id:"(null)",
+		(m->window_role)?m->window_role:"(null)");
+	if (m->wm_command)
+	{
+		for (i = 0; i < m->wm_command_count; i++)
+		{
+			fprintf(stderr," %s", m->wm_command[i]);
+		}
+		fprintf(stderr,",");
+	}
+	else
+	{
+		fprintf(stderr," no_wmc,");
+	}
+	fprintf(stderr," %d)\n\n", IS_NAME_CHANGED(m));
+#endif
+
 	if (client_id)
 	{
 		XFree(client_id);
@@ -581,12 +659,6 @@ static Bool matchWin(FvwmWindow *w, Match *m)
 		XFreeStringList (wm_command);
 	}
 
-#ifdef FVWM_DEBUG_DEVEL
-	fprintf(stderr, "\twin(%s, %s, %s, %d)\n[%d]\tmat(%s, %s, %s, %d)\n\n",
-		w->class.res_name, w->class.res_class, w->name,
-		IS_NAME_CHANGED(w), found, m->res_name, m->res_class,
-		m->wm_name, IS_NAME_CHANGED(m));
-#endif
 	return found;
 }
 
@@ -609,13 +681,11 @@ static int saveStateFile(char *filename)
 	fprintf(f, "# Normally, you must never delete this file,"
 		" it will be auto-deleted.\n\n");
 
-#ifdef SESSION
-	if (goingToRestart)
+	if (SessionSupport && goingToRestart)
 	{
 		fprintf(f, "[REAL_STATE_FILENAME] %s\n", realStateFilename);
 		goingToRestart = False;  /* not needed */
 	}
-#endif
 
 	success = doPreserveState?
 		SaveVersionInfo(f) && SaveWindowStates(f) && SaveGlobalState(f):
@@ -624,37 +694,37 @@ static int saveStateFile(char *filename)
 	if (fclose(f) != 0)
 		return 0;
 
-#ifdef FVWM_DEBUG_DEVEL
+#ifdef FVWM_SM_DEBUG_FILES
 	system(CatString3(
 		       "mkdir -p /tmp/fs-save; cp ", filename,
 		       " /tmp/fs-save"));
 #endif
-#ifdef FVWM_DEBUG_MSGS
-	fprintf(stderr, "\tSaving %s\n", filename);
+#if defined(FVWM_SM_DEBUG_PROTO) || defined(FVWM_SM_DEBUG_FILES)
+	fprintf(stderr, "[FVWM_SMDEBUG] Saving %s\n", filename);
 #endif
 
 	return success;
 }
 
-#ifndef SESSION
-#define setSmProperties(a, b, c)
-#else
 static void
-setSmProperties(SmcConn sm_conn, char *filename, char hint)
+setSmProperties(FSmcConn sm_conn, char *filename, char hint)
 {
-	SmProp prop1, prop2, prop3, prop4, prop5, prop6, *props[6];
-	SmPropValue prop1val, prop2val, prop3val, prop4val;
-	/*#ifdef XSM_BUGGY_DISCARD_COMMAND*/
-	SmPropValue prop6val;
-	/*#endif*/
+	FSmProp prop1, prop2, prop3, prop4, prop5, prop6, prop7, *props[7];
+	FSmPropValue prop1val, prop2val, prop3val, prop4val, prop7val;
 	struct passwd *pwd;
 	char *user_id;
+	char screen_num[32];
 	int numVals, i, priority = 30;
-	Bool xsmDetected;
+	Bool xsmDetected = False;
 
-#ifdef FVWM_DEBUG_MSGS
-	fprintf(stderr, "\t[setSmProperties] state filename: %s%s\n", filename,
-		sm_conn? "": " - not connected");
+	if (!SessionSupport)
+	{
+		return;
+	}
+
+#ifdef FVWM_SM_DEBUG_PROTO
+	fprintf(stderr, "[FVWM_SMDEBUG][setSmProperties] state filename: %s%s\n",
+		(filename)? filename:"(null)", sm_conn? "": " - not connected");
 #endif
 
 	if (!sm_conn)
@@ -665,122 +735,148 @@ setSmProperties(SmcConn sm_conn, char *filename, char hint)
 	pwd = getpwuid (getuid());
 	user_id = pwd->pw_name;
 
-	prop1.name = SmProgram;
-	prop1.type = SmARRAY8;
+	prop1.name = FSmProgram;
+	prop1.type = FSmARRAY8;
 	prop1.num_vals = 1;
 	prop1.vals = &prop1val;
 	prop1val.value = g_argv[0];
 	prop1val.length = strlen (g_argv[0]);
 
-	prop2.name = SmUserID;
-	prop2.type = SmARRAY8;
+	prop2.name = FSmUserID;
+	prop2.type = FSmARRAY8;
 	prop2.num_vals = 1;
 	prop2.vals = &prop2val;
-	prop2val.value = (SmPointer) user_id;
+	prop2val.value = (FSmPointer) user_id;
 	prop2val.length = strlen (user_id);
 
-	prop3.name = SmRestartStyleHint;
-	prop3.type = SmCARD8;
+	prop3.name = FSmRestartStyleHint;
+	prop3.type = FSmCARD8;
 	prop3.num_vals = 1;
 	prop3.vals = &prop3val;
-	prop3val.value = (SmPointer) &hint;
+	prop3val.value = (FSmPointer) &hint;
 	prop3val.length = 1;
 
 	prop4.name = "_GSM_Priority";
-	prop4.type = SmCARD8;
+	prop4.type = FSmCARD8;
 	prop4.num_vals = 1;
 	prop4.vals = &prop4val;
-	prop4val.value = (SmPointer) &priority;
+	prop4val.value = (FSmPointer) &priority;
 	prop4val.length = 1;
 
-	prop5.name = SmRestartCommand;
-	prop5.type = SmLISTofARRAY8;
-
-	prop5.vals = (SmPropValue *)malloc(
-		(g_argc + 7) * sizeof (SmPropValue));
+	sprintf(screen_num, "%d", (int)Scr.screen);
 
+	prop5.name = FSmCloneCommand;
+	prop5.type = FSmLISTofARRAY8;
+	prop5.vals = (FSmPropValue *)malloc((g_argc + 2) * sizeof (FSmPropValue));
 	numVals = 0;
-
 	for (i = 0; i < g_argc; i++)
 	{
 		if (strcmp (g_argv[i], "-clientId") == 0 ||
 		    strcmp (g_argv[i], "-restore") == 0 ||
-		    strcmp (g_argv[i], "-d") == 0)
+		    strcmp (g_argv[i], "-d") == 0 ||
+		    (strcmp (g_argv[i], "-s") == 0 && i+1 < g_argc &&
+		     g_argv[i+1][0] != '-'))
 		{
 			i++;
 		}
 		else if (strcmp (g_argv[i], "-s") != 0)
 		{
-			prop5.vals[numVals].value = (SmPointer) g_argv[i];
+			prop5.vals[numVals].value = (FSmPointer) g_argv[i];
 			prop5.vals[numVals++].length = strlen (g_argv[i]);
 		}
 	}
 
-	prop5.vals[numVals].value = (SmPointer) "-d";
+	prop5.vals[numVals].value = (FSmPointer) "-s";
 	prop5.vals[numVals++].length = 2;
 
-	prop5.vals[numVals].value = (SmPointer) XDisplayString (dpy);
-	prop5.vals[numVals++].length = strlen (XDisplayString (dpy));
+	prop5.vals[numVals].value = (FSmPointer) screen_num;
+	prop5.vals[numVals++].length = strlen (screen_num);
 
-	prop5.vals[numVals].value = (SmPointer) "-s";
-	prop5.vals[numVals++].length = 2;
-
-	prop5.vals[numVals].value = (SmPointer) "-clientId";
-	prop5.vals[numVals++].length = 9;
-
-	prop5.vals[numVals].value = (SmPointer) sm_client_id;
-	prop5.vals[numVals++].length = strlen (sm_client_id);
-
-	prop5.vals[numVals].value = (SmPointer) "-restore";
-	prop5.vals[numVals++].length = 8;
-
-	prop5.vals[numVals].value = (SmPointer) filename;
-	prop5.vals[numVals++].length = strlen (filename);
 
 	prop5.num_vals = numVals;
 
-	prop6.name = SmDiscardCommand;
-
-	/* migo (15-Jul-1999): IMHO, XSM_BUGGY_DISCARD_COMMAND must be deleted
-	 * at all.  Is there a better way to detect xsm, then requiring to set
-	 * environment? */
-	xsmDetected =
-#ifdef XSM_BUGGY_DISCARD_COMMAND
-		1
-#else
-		StrEquals(getenv("SESSION_MANAGER_NAME"), "xsm")
-#endif
-		;
-
-	/*#ifdef XSM_BUGGY_DISCARD_COMMAND*/
-	if (xsmDetected)
+	if (filename)
 	{
-		/* the protocol spec says that the discard command
-		   should be LISTofARRAY8 on posix systems, but xsm
-		   demands that it be ARRAY8.
-		*/
-		char *discardCommand = alloca(
-			(10 + strlen(filename)) * sizeof(char));
-		sprintf (discardCommand, "rm -f '%s'", filename);
-		prop6.type = SmARRAY8;
-		prop6.num_vals = 1;
-		prop6.vals = &prop6val;
-		prop6val.value = (SmPointer) discardCommand;
-		prop6val.length = strlen (discardCommand);
-		/*#else*/
-	}
-	else
-	{
-		prop6.type = SmLISTofARRAY8;
-		prop6.num_vals = 3;
-		prop6.vals = (SmPropValue *) malloc (3 * sizeof (SmPropValue));
-		prop6.vals[0].value = "rm";
-		prop6.vals[0].length = 2;
-		prop6.vals[1].value = "-f";
-		prop6.vals[1].length = 2;
-		prop6.vals[2].value = filename;
-		prop6.vals[2].length = strlen (filename);
-		/*#endif*/
+		prop6.name = FSmRestartCommand;
+		prop6.type = FSmLISTofARRAY8;
+
+		prop6.vals = (FSmPropValue *)malloc(
+			(g_argc + 6) * sizeof (FSmPropValue));
+
+		numVals = 0;
+
+		for (i = 0; i < g_argc; i++)
+		{
+			if (strcmp (g_argv[i], "-clientId") == 0 ||
+			    strcmp (g_argv[i], "-restore") == 0 ||
+			    strcmp (g_argv[i], "-d") == 0 ||
+			    (strcmp (g_argv[i], "-s") == 0 && i+1 < g_argc &&
+			     g_argv[i+1][0] != '-'))
+			{
+				i++;
+			}
+			else if (strcmp (g_argv[i], "-s") != 0)
+			{
+				prop6.vals[numVals].value =
+					(FSmPointer) g_argv[i];
+				prop6.vals[numVals++].length =
+					strlen (g_argv[i]);
+			}
+		}
+
+		prop6.vals[numVals].value = (FSmPointer) "-s";
+		prop6.vals[numVals++].length = 2;
+
+		prop6.vals[numVals].value = (FSmPointer) screen_num;
+		prop6.vals[numVals++].length = strlen (screen_num);
+
+		prop6.vals[numVals].value = (FSmPointer) "-clientId";
+		prop6.vals[numVals++].length = 9;
+
+		prop6.vals[numVals].value = (FSmPointer) sm_client_id;
+		prop6.vals[numVals++].length = strlen (sm_client_id);
+
+		prop6.vals[numVals].value = (FSmPointer) "-restore";
+		prop6.vals[numVals++].length = 8;
+
+		prop6.vals[numVals].value = (FSmPointer) filename;
+		prop6.vals[numVals++].length = strlen (filename);
+
+		prop6.num_vals = numVals;
+
+		prop7.name = FSmDiscardCommand;
+
+		xsmDetected = StrEquals(getenv("SESSION_MANAGER_NAME"), "xsm");
+
+		if (xsmDetected)
+		{
+			/* the protocol spec says that the discard command
+			   should be LISTofARRAY8 on posix systems, but xsm
+			   demands that it be ARRAY8.
+			*/
+			char *discardCommand = alloca(
+				(10 + strlen(filename)) * sizeof(char));
+			sprintf (discardCommand, "rm -f '%s'", filename);
+			prop7.type = FSmARRAY8;
+			prop7.num_vals = 1;
+			prop7.vals = &prop7val;
+			prop7val.value = (FSmPointer) discardCommand;
+			prop7val.length = strlen (discardCommand);
+		}
+		else
+		{
+			prop7.type = FSmLISTofARRAY8;
+			prop7.num_vals = 3;
+			prop7.vals =
+				(FSmPropValue *) malloc (
+					3 * sizeof (FSmPropValue));
+			prop7.vals[0].value = "rm";
+			prop7.vals[0].length = 2;
+			prop7.vals[1].value = "-f";
+			prop7.vals[1].length = 2;
+			prop7.vals[2].value = filename;
+			prop7.vals[2].length = strlen (filename);
+		}
 	}
 
 	props[0] = &prop1;
@@ -788,59 +884,102 @@ setSmProperties(SmcConn sm_conn, char *filename, char hint)
 	props[2] = &prop3;
 	props[3] = &prop4;
 	props[4] = &prop5;
-	props[5] = &prop6;
 
-	SmcSetProperties (sm_conn, 6, props);
+	if (filename)
+	{
+		props[5] = &prop6;
+		props[6] = &prop7;
+		FSmcSetProperties (sm_conn, 7, props);
 
+		free ((char *) prop6.vals);
+		if (!xsmDetected)
+		{
+			free ((char *) prop7.vals);
+		}
+	}
+	else
+	{
+		FSmcSetProperties (sm_conn, 5, props);
+	}
 	free ((char *) prop5.vals);
-	/*#ifndef XSM_BUGGY_DISCARD_COMMAND*/
-	if (!xsmDetected) free ((char *) prop6.vals);
-	/*#endif*/
 }
-#endif
 
-#ifdef SESSION
 static void
-callback_save_yourself2(SmcConn sm_conn, SmPointer client_data)
+callback_save_yourself2(FSmcConn sm_conn, FSmPointer client_data)
 {
 	Bool success = 0;
-	char *filename = getUniqueStateFilename();
+	char *filename;
 
-#ifdef FVWM_DEBUG_MSGS
-	fprintf(stderr, "\t[callback_save_yourself2]\n");
+
+	if (!SessionSupport)
+	{
+		return;
+	}
+
+	filename = getUniqueStateFilename();
+#ifdef FVWM_SM_DEBUG_PROTO
+	fprintf(stderr, "[FVWM_SMDEBUG][callback_save_yourself2]\n");
 #endif
 
 	success = saveStateFile(filename);
 	if (success)
 	{
-		setSmProperties(sm_conn, filename, SmRestartIfRunning);
+		setSmProperties(sm_conn, filename, FSmRestartIfRunning);
 		setRealStateFilename(filename);
 	}
 	free(filename);
 
-	SmcSaveYourselfDone (sm_conn, success);
+	FSmcSaveYourselfDone (sm_conn, success);
 	sent_save_done = 1;
 }
 
 static void
-callback_save_yourself(SmcConn sm_conn, SmPointer client_data,
+callback_save_yourself(FSmcConn sm_conn, FSmPointer client_data,
 		       int save_style, Bool shutdown, int interact_style,
 		       Bool fast)
 {
-#ifdef FVWM_DEBUG_MSGS
-	fprintf(stderr, "\t[callback_save_yourself] (save=%d, shut=%d,"
-		" intr=%d, fast=%d)\n", save_style, shutdown, interact_style,
-		fast);
+
+	if (!SessionSupport)
+	{
+		return;
+	}
+
+#ifdef FVWM_SM_DEBUG_PROTO
+	fprintf(stderr, "[FVWM_SMDEBUG][callback_save_yourself] "
+		"(save=%d, shut=%d, intr=%d, fast=%d)\n",
+		save_style, shutdown, interact_style, fast);
 #endif
 
-	if (!SmcRequestSaveYourselfPhase2(
+	if (save_style == FSmSaveGlobal)
+	{
+		/* nothing to do */
+#ifdef FVWM_SM_DEBUG_PROTO
+		fprintf(stderr, "[FVWM_SMDEBUG][callback_save_yourself] "
+		"Global Save type ... do nothing\n");
+#endif
+		FSmcSaveYourselfDone (sm_conn, True);
+		sent_save_done = 1;
+		return;
+
+	}
+#ifdef FVWM_SM_DEBUG_PROTO
+	fprintf(stderr, "[FVWM_SMDEBUG][callback_save_yourself] "
+		"Both or Local save type, going to phase 2 ...");
+#endif
+	if (!FSmcRequestSaveYourselfPhase2(
 		    sm_conn, callback_save_yourself2, NULL))
 	{
-		SmcSaveYourselfDone (sm_conn, False);
+		FSmcSaveYourselfDone (sm_conn, False);
 		sent_save_done = 1;
+#ifdef FVWM_SM_DEBUG_PROTO
+		fprintf(stderr, " failed!\n");
+#endif
 	}
 	else
 	{
+#ifdef FVWM_SM_DEBUG_PROTO
+		fprintf(stderr, " OK\n");
+#endif
 		sent_save_done = 0;
 	}
 
@@ -848,13 +987,22 @@ callback_save_yourself(SmcConn sm_conn, SmPointer client_data,
 }
 
 static void
-callback_die(SmcConn sm_conn, SmPointer client_data)
+callback_die(FSmcConn sm_conn, FSmPointer client_data)
 {
-#ifdef FVWM_DEBUG_MSGS
-	fprintf(stderr, "\t[callback_die]\n");
+
+	if (!SessionSupport)
+	{
+		return;
+	}
+
+#ifdef FVWM_SM_DEBUG_PROTO
+	fprintf(stderr, "[FVWM_SMDEBUG][callback_die]\n");
 #endif
 
-	SmcCloseConnection(sm_conn, 0, NULL);
+	if (FSmcCloseConnection(sm_conn, 0, NULL) != FSmcClosedNow)
+	{
+		/* go a head any way ? */
+	}
 	sm_fd = -1;
 
 	if (master_pid != getpid())
@@ -865,25 +1013,34 @@ callback_die(SmcConn sm_conn, SmPointer client_data)
 }
 
 static void
-callback_save_complete(SmcConn sm_conn, SmPointer client_data)
+callback_save_complete(FSmcConn sm_conn, FSmPointer client_data)
 {
-#ifdef FVWM_DEBUG_MSGS
-	fprintf(stderr, "\t[callback_save_complete]\n");
+	if (!SessionSupport)
+	{
+		return;
+	}
+#ifdef FVWM_SM_DEBUG_PROTO
+	fprintf(stderr, "[FVWM_SMDEBUG][callback_save_complete]\n");
 #endif
 
 	return;
 }
 
 static void
-callback_shutdown_cancelled(SmcConn sm_conn, SmPointer client_data)
+callback_shutdown_cancelled(FSmcConn sm_conn, FSmPointer client_data)
 {
-#ifdef FVWM_DEBUG_MSGS
-	fprintf(stderr, "\t[callback_shutdown_cancelled]\n");
+	if (!SessionSupport)
+	{
+		return;
+	}
+
+#ifdef FVWM_SM_DEBUG_PROTO
+	fprintf(stderr, "[FVWM_SMDEBUG][callback_shutdown_cancelled]\n");
 #endif
 
 	if (!sent_save_done)
 	{
-		SmcSaveYourselfDone(sm_conn, False);
+		FSmcSaveYourselfDone(sm_conn, False);
 		sent_save_done = 1;
 	}
 
@@ -892,8 +1049,13 @@ callback_shutdown_cancelled(SmcConn sm_conn, SmPointer client_data)
 
 /* the following is taken from xsm */
 static void
-MyIoErrorHandler(IceConn ice_conn)
+MyIoErrorHandler(FIceConn ice_conn)
 {
+	if (!SessionSupport)
+	{
+		return;
+	}
+
 	if (prev_handler)
 	{
 		(*prev_handler) (ice_conn);
@@ -905,10 +1067,15 @@ MyIoErrorHandler(IceConn ice_conn)
 static void
 InstallIOErrorHandler(void)
 {
-	IceIOErrorHandler default_handler;
+	FIceIOErrorHandler default_handler;
 
-	prev_handler = IceSetIOErrorHandler (NULL);
-	default_handler = IceSetIOErrorHandler (MyIoErrorHandler);
+	if (!SessionSupport)
+	{
+		return;
+	}
+
+	prev_handler = FIceSetIOErrorHandler (NULL);
+	default_handler = FIceSetIOErrorHandler (MyIoErrorHandler);
 	if (prev_handler == default_handler)
 	{
 		prev_handler = NULL;
@@ -916,9 +1083,8 @@ InstallIOErrorHandler(void)
 
 	return;
 }
-#endif
 
-/* ---------------------------- interface functions ------------------------- */
+/* ---------------------------- interface functions ------------------------ */
 
 void
 LoadGlobalState(char *filename)
@@ -945,95 +1111,92 @@ LoadGlobalState(char *filename)
 	{
 		i1 = 0; i2 = 0; i3 = 0; i4 = 0; i5 = 0; i6 = 0;
 		sscanf(s, "%4000s", s1);
-#ifdef SESSION
 		/* If we are restarting, [REAL_STATE_FILENAME] points
 		 * to the file containing the true session state. */
-		if (!strcmp(s1, "[REAL_STATE_FILENAME]"))
+		if (SessionSupport && !strcmp(s1, "[REAL_STATE_FILENAME]"))
 		{
-			/* migo: temporarily (?) moved to LoadWindowStates
-			 * (trick for gnome-session)
-			 sscanf(s, "%*s %s", s2);
-			 setSmProperties (sm_conn, s2, SmRestartIfRunning);
-			 setRealStateFilename(s2);
+			/* migo: temporarily (?) moved to
+			   LoadWindowStates (trick for gnome-session)
+			   sscanf(s, "%*s %s", s2);
+			   setFSmProperties (sm_conn, s2, FSmRestartIfRunning);
+			   setRealStateFilename(s2);
 			*/
 		}
-		else
-#endif /* SESSION */
-			if (!strcmp(s1, "[DESKTOP]"))
-			{
-				sscanf(s, "%*s %i", &i1);
-				goto_desk(i1);
-			}
-			else if (!strcmp(s1, "[VIEWPORT]"))
-			{
-				sscanf(s, "%*s %i %i %i %i", &i1, &i2, &i3,
-				       &i4);
-				/* migo: we don't want to lose DeskTopSize in
-				 * configurations, and it does not work well
-				 * anyways - Gnome is not updated
-				 Scr.VxMax = i3;
-				 Scr.VyMax = i4;
-				*/
-				MoveViewport(i1, i2, True);
-			}
+		else if (!strcmp(s1, "[DESKTOP]"))
+		{
+			sscanf(s, "%*s %i", &i1);
+			goto_desk(i1);
+		}
+		else if (!strcmp(s1, "[VIEWPORT]"))
+		{
+			sscanf(s, "%*s %i %i %i %i", &i1, &i2, &i3,
+			       &i4);
+			/* migo: we don't want to lose DeskTopSize in
+			 * configurations, and it does not work well
+			 * anyways - Gnome is not updated
+			 Scr.VxMax = i3;
+			 Scr.VyMax = i4;
+			*/
+			MoveViewport(i1, i2, True);
+		}
 #if 0
 		/* migo (08-Dec-1999): we don't want to eliminate .fvwm2rc for
 		 * now */
-			else if (/*!Restarting*/ 0)
+		else if (/*!Restarting*/ 0)
+		{
+			/* Matthias: We don't want to restore too much
+			 * state if we are restarting, since that
+			 * would make restarting useless for rereading
+			 * changed rc files. */
+			if (!strcmp(s1, "[SCROLL]"))
 			{
-				/* Matthias: We don't want to restore too much
-				 * state if we are restarting, since that
-				 * would make restarting useless for rereading
-				 * changed rc files. */
-				if (!strcmp(s1, "[SCROLL]"))
+				sscanf(s, "%*s %i %i %i %i %i %i", &i1,
+				       &i2, &i3, &i4, &i5, &i6);
+				Scr.EdgeScrollX = i1;
+				Scr.EdgeScrollY = i2;
+				Scr.ScrollResistance = i3;
+				Scr.MoveResistance = i4;
+				if (i5)
 				{
-					sscanf(s, "%*s %i %i %i %i %i %i", &i1,
-					       &i2, &i3, &i4, &i5, &i6);
-					Scr.EdgeScrollX = i1;
-					Scr.EdgeScrollY = i2;
-					Scr.ScrollResistance = i3;
-					Scr.MoveResistance = i4;
-					if (i5)
-					{
-						Scr.flags.edge_wrap_x = 1;
-					}
-					else
-					{
-						Scr.flags.edge_wrap_x = 0;
-					}
-					if (i6)
-					{
-						Scr.flags.edge_wrap_y = 1;
-					}
-					else
-					{
-						Scr.flags.edge_wrap_y = 0;
-					}
+					Scr.flags.edge_wrap_x = 1;
 				}
-				else if (!strcmp(s1, "[SNAP]"))
+				else
 				{
-					sscanf(s, "%*s %i %i %i %i", &i1, &i2,
-					       &i3, &i4);
-					Scr.SnapAttraction = i1;
-					Scr.SnapMode = i2;
-					Scr.SnapGridX = i3;
-					Scr.SnapGridY = i4;
+					Scr.flags.edge_wrap_x = 0;
 				}
-				else if (!strcmp(s1, "[MISC]"))
+				if (i6)
 				{
-					sscanf(s, "%*s %i %i %i", &i1, &i2,
-					       &i3);
-					Scr.ClickTime = i1;
-					Scr.ColormapFocus = i2;
-					Scr.ColorLimit = i3;
+					Scr.flags.edge_wrap_y = 1;
 				}
-				else if (!strcmp(s1, "[STYLE]"))
+				else
 				{
-					sscanf(s, "%*s %i %i", &i1, &i2);
-					Scr.gs.EmulateMWM = i1;
-					Scr.gs.EmulateWIN = i2;
+					Scr.flags.edge_wrap_y = 0;
 				}
 			}
+			else if (!strcmp(s1, "[SNAP]"))
+			{
+				sscanf(s, "%*s %i %i %i %i", &i1, &i2,
+				       &i3, &i4);
+				Scr.SnapAttraction = i1;
+				Scr.SnapMode = i2;
+				Scr.SnapGridX = i3;
+				Scr.SnapGridY = i4;
+			}
+			else if (!strcmp(s1, "[MISC]"))
+			{
+				sscanf(s, "%*s %i %i %i", &i1, &i2,
+				       &i3);
+				Scr.ClickTime = i1;
+				Scr.ColormapFocus = i2;
+				Scr.ColorLimit = i3;
+			}
+			else if (!strcmp(s1, "[STYLE]"))
+			{
+				sscanf(s, "%*s %i %i", &i1, &i2);
+				Scr.gs.EmulateMWM = i1;
+				Scr.gs.EmulateWIN = i2;
+			}
+		}
 #endif
 	}
 	fclose(f);
@@ -1073,10 +1236,10 @@ LoadWindowStates(char *filename)
 		return;
 	}
 
-#ifdef FVWM_DEBUG_MSGS
-	fprintf(stderr, "\tLoading %s\n", filename);
+#if defined(FVWM_SM_DEBUG_PROTO) ||  defined(FVWM_SM_DEBUG_FILES)
+	fprintf(stderr, "[FVWM_SMDEBUG] Loading %s\n", filename);
 #endif
-#ifdef FVWM_DEBUG_DEVEL
+#ifdef FVWM_SM_DEBUG_FILES
 	system(CatString3(
 		       "mkdir -p /tmp/fs-load; cp ", filename,
 		       " /tmp/fs-load"));
@@ -1086,15 +1249,11 @@ LoadWindowStates(char *filename)
 	{
 		n = 0;
 		sscanf(s, "%4000s%n", s1, &n);
-		/* migo: temporarily */
-		if (
-#ifndef SESSION
-			0 &&
-#endif /* SESSION */
-			!strcmp(s1, "[REAL_STATE_FILENAME]"))
+		if (!SessionSupport /* migo: temporarily */ &&
+		    !strcmp(s1, "[REAL_STATE_FILENAME]"))
 		{
 			sscanf(s, "%*s %s", s1);
-			setSmProperties (sm_conn, s1, SmRestartIfRunning);
+			setSmProperties (sm_conn, s1, FSmRestartIfRunning);
 			setRealStateFilename(s1);
 		}
 		else if (!strcmp(s1, "[CLIENT]"))
@@ -1268,7 +1427,7 @@ LoadWindowStates(char *filename)
 Bool
 MatchWinToSM(
 	FvwmWindow *ewin, mwtsm_state_args *ret_state_args,
-	initial_window_options_type *win_opts)
+	initial_window_options_t *win_opts)
 {
 	int i;
 
@@ -1281,6 +1440,7 @@ MatchWinToSM(
 		if (!matches[i].used && matchWin(ewin, &matches[i]))
 		{
 			matches[i].used = 1;
+
 			if (!Restarting)
 			{
 				/* We don't want to restore too much state if
@@ -1324,6 +1484,7 @@ MatchWinToSM(
 					setup_visible_name(ewin, False);
 				}
 			}
+			SET_NAME_CHANGED(ewin,IS_NAME_CHANGED(&(matches[i])));
 			SET_PLACED_WB3(ewin,IS_PLACED_WB3(&(matches[i])));
 			SET_PLACED_BY_FVWM(
 				ewin, IS_PLACED_BY_FVWM(&(matches[i])));
@@ -1408,13 +1569,13 @@ void
 RestartInSession (char *filename, Bool isNative, Bool _doPreserveState)
 {
 	doPreserveState = _doPreserveState;
-#ifdef SESSION
-	if (sm_conn && isNative)
+
+	if (SessionSupport && sm_conn && isNative)
 	{
 		goingToRestart = True;
 
 		saveStateFile(filename);
-		setSmProperties(sm_conn, filename, SmRestartImmediately);
+		setSmProperties(sm_conn, filename, FSmRestartImmediately);
 
 		MoveViewport(0, 0, False);
 		Reborder();
@@ -1422,26 +1583,33 @@ RestartInSession (char *filename, Bool isNative, Bool _doPreserveState)
 		CloseICCCM2();
 		XCloseDisplay(dpy);
 
-		SmcCloseConnection(sm_conn, 0, NULL);
+		if (!FSmcCloseConnection(sm_conn, 0, NULL) != FSmcClosedNow)
+		{
+			/* go a head any way ? */
+		}
 
-#ifdef FVWM_DEBUG_MSGS
-		fprintf(stderr, "[FVWM]: Exiting, now SM must restart us.\n");
+#ifdef  FVWM_SM_DEBUG_PROTO
+		fprintf(stderr, "[FVWM_SMDEBUG]: Exiting, now SM must "
+			"restart us.\n");
 #endif
 		/* Close all my pipes */
 		ClosePipes();
 
 		exit(0); /* let the SM restart us */
 	}
-#endif
+
 	saveStateFile(filename);
 	/* return and let Done restart us */
 
 	return;
 }
 
-#ifdef SESSION
 void SetClientID(char *client_id)
 {
+	if (!SessionSupport)
+	{
+		return;
+	}
 	previous_sm_client_id = client_id;
 
 	return;
@@ -1451,9 +1619,18 @@ void
 SessionInit(void)
 {
 	char error_string_ret[4096] = "";
-	static SmPointer context;
-	SmcCallbacks callbacks;
+	FSmPointer context;
+	FSmcCallbacks callbacks;
 
+	if (!SessionSupport)
+	{
+		/* -Wall fixes */
+		MyIoErrorHandler(NULL);
+		callback_save_yourself2(NULL, NULL);
+		return;
+	}
+
+	context = NULL;
 	InstallIOErrorHandler();
 
 	callbacks.save_yourself.callback = callback_save_yourself;
@@ -1463,11 +1640,11 @@ SessionInit(void)
 	callbacks.save_yourself.client_data =
 		callbacks.die.client_data =
 		callbacks.save_complete.client_data =
-		callbacks.shutdown_cancelled.client_data = (SmPointer) NULL;
-	sm_conn = SmcOpenConnection(
-		NULL, &context, SmProtoMajor, SmProtoMinor,
-		SmcSaveYourselfProcMask | SmcDieProcMask |
-		SmcSaveCompleteProcMask | SmcShutdownCancelledProcMask,
+		callbacks.shutdown_cancelled.client_data = (FSmPointer) NULL;
+	sm_conn = FSmcOpenConnection(
+		NULL, &context, FSmProtoMajor, FSmProtoMinor,
+		FSmcSaveYourselfProcMask | FSmcDieProcMask |
+		FSmcSaveCompleteProcMask | FSmcShutdownCancelledProcMask,
 		&callbacks, previous_sm_client_id, &sm_client_id, 4096,
 		error_string_ret);
 	if (!sm_conn)
@@ -1483,13 +1660,22 @@ SessionInit(void)
 				error_string_ret);
 		}
 		sm_fd = -1;
+#ifdef FVWM_SM_DEBUG_PROTO
+		fprintf(stderr,"[FVWM_SMDEBUG] No SM connection\n");
+#endif
 	}
 	else
 	{
-		sm_fd = IceConnectionNumber(SmcGetIceConnection(sm_conn));
+		sm_fd = FIceConnectionNumber(FSmcGetIceConnection(sm_conn));
+#ifdef FVWM_SM_DEBUG_PROTO
+		fprintf(stderr,"[FVWM_SMDEBUG] Connectecd to a SM\n");
+#endif
 		setInitFunctionName(0, "SessionInitFunction");
 		setInitFunctionName(1, "SessionRestartFunction");
 		setInitFunctionName(2, "SessionExitFunction");
+		/* basically to restet our restart style hint after a
+		 * restart */
+		setSmProperties(sm_conn, NULL, FSmRestartIfRunning);
 	}
 
 	return;
@@ -1498,14 +1684,22 @@ SessionInit(void)
 void
 ProcessICEMsgs(void)
 {
-	IceProcessMessagesStatus status;
+	FIceProcessMessagesStatus status;
 
+	if (!SessionSupport)
+	{
+		return;
+	}
+
+#ifdef FVWM_SM_DEBUG_PROTO
+	fprintf(stderr,"[FVWM_SMDEBUG][ProcessICEMsgs] %i\n", (int)sm_fd);
+#endif
 	if (sm_fd < 0)
 	{
 		return;
 	}
-	status = IceProcessMessages(SmcGetIceConnection(sm_conn), NULL, NULL);
-	if (status == IceProcessMessagesIOError)
+	status = FIceProcessMessages(FSmcGetIceConnection(sm_conn), NULL, NULL);
+	if (status == FIceProcessMessagesIOError)
 	{
 		fvwm_msg(ERR, "ProcessICEMSGS",
 			 "Connection to session manager lost\n");
@@ -1529,6 +1723,11 @@ int getSmPid(void)
 	const char *smPidStrPtr;
 	int smPid = 0;
 
+	if (!SessionSupport)
+	{
+		return 0;
+	}
+
 	if (!sessionManagerVar)
 	{
 		return 0;
@@ -1549,21 +1748,23 @@ int getSmPid(void)
 
 	return smPid;
 }
-#endif
 
 /*
  * quitSession - hopefully shutdowns the session
  */
 Bool quitSession(void)
 {
-#ifdef SESSION
+	if (!SessionSupport)
+	{
+		return False;
+	}
 	if (!sm_conn)
 	{
 		return False;
 	}
 
-	SmcRequestSaveYourself(
-		sm_conn, SmSaveLocal, True /* shutdown */, SmInteractStyleAny,
+	FSmcRequestSaveYourself(
+		sm_conn, FSmSaveLocal, True /* shutdown */, FSmInteractStyleAny,
 		False /* fast */, True /* global */);
 	return True;
 
@@ -1574,9 +1775,6 @@ Bool quitSession(void)
 	  if (!smPid) return False;
 	  return kill(smPid, SIGTERM) == 0? True: False;
 	*/
-#else
-	return False;
-#endif
 }
 
 /*
@@ -1584,14 +1782,17 @@ Bool quitSession(void)
  */
 Bool saveSession(void)
 {
-#ifdef SESSION
+	if (!SessionSupport)
+	{
+		return False;
+	}
 	if (!sm_conn)
 	{
 		return False;
 	}
 
-	SmcRequestSaveYourself(
-		sm_conn, SmSaveBoth, False /* shutdown */, SmInteractStyleAny,
+	FSmcRequestSaveYourself(
+		sm_conn, FSmSaveBoth, False /* shutdown */, FSmInteractStyleAny,
 		False /* fast */, True /* global */);
 	return True;
 
@@ -1602,9 +1803,6 @@ Bool saveSession(void)
 	  if (!smPid) return False;
 	  return kill(smPid, SIGUSR1) == 0? True: False;
 	*/
-#else
-	return False;
-#endif
 }
 
 /*
@@ -1612,14 +1810,18 @@ Bool saveSession(void)
  */
 Bool saveQuitSession(void)
 {
-#ifdef SESSION
+	if (!SessionSupport)
+	{
+		return False;
+	}
+
 	if (!sm_conn)
 	{
 		return False;
 	}
 
-	SmcRequestSaveYourself(
-		sm_conn, SmSaveBoth, True /* shutdown */, SmInteractStyleAny,
+	FSmcRequestSaveYourself(
+		sm_conn, FSmSaveBoth, True /* shutdown */, FSmInteractStyleAny,
 		False /* fast */, True /* global */);
 	return True;
 
@@ -1631,12 +1833,9 @@ Bool saveQuitSession(void)
 	  if (quitSession() == False) return False;
 	  return True;
 	*/
-#else
-	return False;
-#endif
 }
 
-/* ---------------------------- builtin commands ---------------------------- */
+/* ---------------------------- builtin commands --------------------------- */
 
 void CMD_QuitSession(F_CMD_ARGS)
 {
