@@ -1,18 +1,4 @@
-/* This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- */
-
+/* -*-c-*- */
 #ifndef FVWMFORM_H
 #define FVWMFORM_H
 
@@ -24,6 +10,7 @@
 
  */
 
+#include "libs/Flocale.h"               /* for font definition stuff */
 /*
  * This next stuff should be more specific and customizable.
  * For example padVText (above) was one of the things TXT_SPC
@@ -43,12 +30,14 @@
 #define ITEMS_PER_EXPANSION 32
 #define CHOICES_PER_SEL_EXPANSION 8
 #define BUTTON_COMMAND_EXPANSION 8
+#define TIMEOUT_COMMAND_EXPANSION 8
 
 #define I_TEXT          1
 #define I_INPUT         2
 #define I_SELECT        3
 #define I_CHOICE        4
 #define I_BUTTON        5
+#define I_TIMEOUT       6
 
 #define IS_SINGLE       1
 #define IS_MULTIPLE     2
@@ -64,14 +53,14 @@ typedef struct _drawtable {
   char *dt_color_names[4];
   int dt_used;                          /* 1=used as text, 2 used as item */
   unsigned long dt_colors[6];             /* text fore/back
-                                           button/choice/input fore/back
-                                           hilite and shadow (item only)
-                                           Note, need all 6 because choices
-                                           use them all */
+					   button/choice/input fore/back
+					   hilite and shadow (item only)
+					   Note, need all 6 because choices
+					   use them all */
   GC dt_GC;                             /* graphic ctx used for text */
   GC dt_item_GC;                        /* graphic ctx used for graphics */
-  Font dt_font;
-  XFontStruct *dt_font_struct;
+  FlocaleFont *dt_Ffont;                /* Fvwm font structure */
+  FlocaleWinString *dt_Fstr;            /* Fvwm string (handles multibyte) */
 } DrawTable;
 
 /* An "item" is something in a form. Part of the structure is common
@@ -104,7 +93,6 @@ typedef union _item {
     char *blanks;          /* blank string */
     int size;              /* input field size */
     int left;              /* position of the left-most displayed char */
-    int o_cursor;          /* store relative cursor position */
     union _item *next_input;            /* a ring of input fields */
     union _item *prev_input;            /* for tabbing */
     int value_history_count;            /* count of input history */
@@ -137,6 +125,13 @@ typedef union _item {
     int button_array_size;              /* current size of next array */
     char **commands;    /* Fvwm command to execute */
   } button;
+  struct {              /* I_TIMEOUT */
+    struct _head head;
+    int timeleft;       /* seconds left on timer */
+    int len;            /* text length */
+    char *text;         /* text string */
+    char *command;    /* Fvwm command(s) to execute */
+  } timeout;
 } Item;
 
 #define L_LEFT        1
@@ -223,8 +218,8 @@ enum { c_bg, c_fg, c_item_bg, c_item_fg, c_itemlo, c_itemhi };
 extern char *color_names[4];
 extern char bg_state;
 extern char endDefaultsRead;
-enum { f_text, f_input, f_button };
-extern char *font_names[3];
+enum { f_text, f_input, f_button, f_timeout };
+extern char *font_names[4];
 extern char *screen_background_color;
 
 extern int colorset;
@@ -252,15 +247,18 @@ extern int Channel[2];
 /* From FvwmAnimate end */
 
 /* prototypes */
-void ReadXServer();                     /* ReadXServer.c */
-void RedrawText(Item *item);            /* FvwmForm.c */
-void RedrawItem (Item *item, int click); /* FvwmForm.c */
-void UpdateRootTransapency(void);        /* FvwmForm.c */
-void DoCommand (Item *cmd);             /* FvwmForm.c */
-int FontWidth (XFontStruct *xfs);       /* FvwmForm.c */
-void RedrawFrame ();                    /* FvwmForm.c */
+void ReadXServer();                      /* ReadXServer.c */
+void RedrawText(Item *item);             /* FvwmForm.c */
+void RedrawTimeout(Item *item);          /* FvwmForm.c */
+void RedrawItem (
+	Item *item, int click, XEvent *pev); /* FvwmForm.c */
+void UpdateRootTransapency(
+	Bool pr_only, Bool do_draw);     /* FvwmForm.c */
+void DoCommand (Item *cmd);              /* FvwmForm.c */
+int FontWidth (XFontStruct *xfs);        /* FvwmForm.c */
+void RedrawFrame (XEvent *pev);          /* FvwmForm.c */
 char * ParseCommand (int, char *, char, int *, char **s); /* ParseCommand.c */
 
-void DeadPipe(int nonsense);            /* FvwmForm.c */
+RETSIGTYPE DeadPipe(int nonsense);            /* FvwmForm.c */
 
 #endif
