@@ -1,3 +1,4 @@
+/* -*-c-*- */
 /* FvwmTaskBar Module for Fvwm.
  *
  *  Copyright 1994,  Mike Finger (mfinger@mermaid.micro.umn.edu or
@@ -18,21 +19,6 @@
  * own risk. Permission to use this program for any purpose is given,
  * as long as the copyright is kept intact. */
 
-/* This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- */
-
 #ifndef FVWMTASKBAR_H
 #define FVWMTASKBAR_H
 
@@ -99,13 +85,13 @@ typedef enum
 	STICK_TASK_BAR = 4
 } tb_alarm_t;
 
-/*************************************************************************
+/*
   Subroutine Prototypes
-**************************************************************************/
+*/
 void EndLessLoop(void);
 void ReadFvwmPipe(void);
 void ProcessMessage(unsigned long type,unsigned long *body);
-void DeadPipe(int nonsense)  __attribute__((noreturn));
+RETSIGTYPE DeadPipe(int nonsense);
 /*
 void Alarm(int nonsense);
 void SetAlarm(int event);
